from pathlib import Path

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer  # noqa


class CallbackCommand(object):

    def path_inventory(self, value: Path):

        if value is None or value == "":
            raise typer.BadParameter("inventory is not directory",
                                     param_hint="--path")

        if value.is_dir():
            return value
        raise typer.BadParameter("inventory is not directory",
                                 param_hint="--path")

    def process_type(self, process: str):

        if process == "" or process is None:
            raise typer.BadParameter("process is not defined",
                                     param_hint="--process")

        if process not in ['src', 'dst']:
            raise typer.BadParameter("Only src or dst process",
                                     param_hint="--process")

        return process

    def output_type(self, output: str, name: str):

        if output not in ['json', 'excel']:
            raise typer.BadParameter("Only json or excel type is valid",
                                     param_hint="--output")

        if output == "excel" and name == "":
            raise typer.BadParameter(
                "a name has not been defined for the excel file",
                param_hint="--name")

        return output



class Data_cibercca:
    def __init__(self, command,date,data_from_device):
        self.command = command
        self.date = date
        self.data_from_device = data_from_device

    def toDBCollection(self):
        return{
            'command': self.command,
            'date': self.date,
            'data_from_device': self.data_from_device
        } 


import json
import os

import pandas as pd


class PingMerge:
    def __init__(self, file_src, file_dst) -> None:
        # --
        # final dataframe compared and validated
        self.data = None

        # --
        # contains the upload data of the files
        self.file_src = file_src
        self.file_dst = file_dst
        self.data_src = None
        self.data_dst = None
        self.file_src_ext = None
        self.file_dst_ext = None

        # --
        # dataframe with the source vrf
        self.df_source = None

        # --
        # dataframe with the destination vrf
        self.df_dest = None

        # --
        # columns for dataframe report baseline ping
        self.columnsDataFrame = [
            "src_device",
            "src_ip_address",
            "src_vrf",
            "src_interface",
            "src_mac",
            "src_interface_ip_address",
            "dst_device",
            "dst_ip_address",
            "dst_vrf",
            "dst_interface",
            "dst_mac",
            "dst_interface_ip_address",
            "initial_ping_percent",
            "final_ping_percent",
            "status"
        ]

        self.BASE_DIR = os.path.dirname(os.path.abspath(__file__))

    # --
    # load the files, source and destination
    def loadFiles(self) -> None:
        """
        load the files, source and destination

        file_src: the source file vrf listing
        file_dst: the destination file vrf listing
        """

        # --
        # load file source and identifier extension
        file = open(self.file_src)
        self.file_src_ext = os.path.splitext(self.file_src)[1]

        if self.file_src_ext == '.json':
            self.data_src = json.load(file)
        else:
            self.data_src = pd.read_excel(self.file_src)

        # --
        # load file destination and identifier extension
        file_dst = open(self.file_dst)
        self.file_dst_ext = os.path.splitext(self.file_dst)[1]

        if self.file_dst_ext == '.json':
            self.data_dst = json.load(file_dst)
        else:
            self.data_dst = pd.read_excel(self.file_dst)

    def json_to_dataframe(self, data):
        """
        json_to_dataframe:

        data: the json data with the vrfs list

        transform the json data to a dataframe
        """

        columns = [
            "device",
            "source",
            "ip_address",
            "vrf",
            "interface",
            "mac",
            "interface_ip_address",
            "percent"
        ]

        final_list = []

        for key in data:
            device = data[key]['data']

            EC_DEVICE_NAME = device['name']
            EC_DEVICE_HOSTNAME = device['hostname']

            try:
                EC_SOURCE = device['source']
            except Exception:
                EC_SOURCE = ''

            # --
            # listar las vrfs asignadas al equipo origen
            # --

            for vrf in device['list_vrf']:
                EC_VRF_NAME = vrf['vrf_name']

                if len(vrf['ips_arp']) > 0:
                    for ip in vrf["ips_arp"]:
                        EC_VRF_INTERFACE = ip['interface']
                        EC_VRF_MAC_ADDRESS = ip['mac']
                        EC_VRF_IP_ADDRESS = ip['ip_address']

                        try:
                            EC_VRF_PERCENT = ip["ping_arp"][0]['percent']
                        except Exception:
                            EC_VRF_PERCENT = ""

                        row = [
                            EC_DEVICE_NAME,
                            EC_SOURCE,
                            EC_DEVICE_HOSTNAME,
                            EC_VRF_NAME,
                            EC_VRF_INTERFACE,
                            EC_VRF_MAC_ADDRESS,
                            EC_VRF_IP_ADDRESS,
                            EC_VRF_PERCENT
                        ]
                        final_list.append(row)

                else:
                    row = [EC_DEVICE_NAME, EC_SOURCE,
                           EC_DEVICE_HOSTNAME, EC_VRF_NAME, "", "", "", ""]
                    final_list.append(row)

            # --
            # si el dispositivo no tiene vrfs asignadas
            if len(device['list_vrf']) == 0:
                row = [EC_DEVICE_NAME, EC_SOURCE,
                       EC_DEVICE_HOSTNAME, "", "", "", "", ""]
                final_list.append(row)

        return pd.DataFrame(final_list, columns=columns)

    def excel_to_dataframe(self, data):
        final_list = []

        columns = [
            "device",
            "source",
            "ip_address",
            "vrf",
            "interface",
            "mac",
            "interface_ip_address",
            "percent"
        ]

        for row in data.itertuples():
            EC_DEVICE_NAME = row[1]
            EC_SOURCE = row[2]
            EC_DEVICE_HOSTNAME = row[3]
            EC_VRF_NAME = row[4]
            EC_VRF_INTERFACE = row[5]
            EC_VRF_MAC_ADDRESS = row[6]
            EC_VRF_IP_ADDRESS = row[7]
            EC_VRF_PERCENT = row[8]

            row = [
                EC_DEVICE_NAME,
                EC_SOURCE,
                EC_DEVICE_HOSTNAME,
                EC_VRF_NAME,
                EC_VRF_INTERFACE,
                EC_VRF_MAC_ADDRESS,
                EC_VRF_IP_ADDRESS,
                EC_VRF_PERCENT
            ]
            final_list.append(row)

        return pd.DataFrame(final_list, columns=columns)

    # --
    # return the data json/excel vrf to dataframe

    def transform(self, data, ext):
        if ext == ".json":
            return self.json_to_dataframe(data)
        else:
            return self.excel_to_dataframe(data)

    def createDataframeSourceOnly(self, origin, dest_device):
        """
        createDataframeSourceOnly:

        origin: the source device vrfs list
        dest_device: the destination device vrfs list

        when the source is validated against the
        destination and exists only in the source
        """  # noqa
        df = pd.DataFrame(
            [
                [
                    origin.device,
                    origin.ip_address,
                    origin.vrf,
                    origin.interface,
                    origin.mac,
                    origin.interface_ip_address,
                    dest_device,
                    "",
                    "",
                    "",
                    "",
                    "",
                    int(origin.percent),
                    0,
                    origin.validate
                ]
            ],
            columns=self.columnsDataFrame
        )

        return df

    def createDataframeSourceDest(self, origin, dest):
        """
        createDataframeSourceDest:

        origin: the source device vrfs list
        dest_device: the destination device vrfs list

        when the destination is validated with the origin and both are found,
        the ping percentage is validated
        """  # noqa
        df = pd.DataFrame(
            [
                [
                    origin.device.values[0],
                    origin.ip_address.values[0],
                    origin.vrf.values[0],
                    origin.interface.values[0],
                    origin.mac.values[0],
                    origin.interface_ip_address.values[0],
                    dest.device,
                    dest.ip_address,
                    dest.vrf,
                    dest.interface,
                    dest.mac,
                    dest.interface_ip_address,
                    int(origin.percent),
                    int(dest.percent),
                    dest.validate
                ]
            ],
            columns=self.columnsDataFrame
        )

        return df

    def createDataframeDestNew(self, dest, source_device):
        """
        createDataframeDestNew:

        origin: the source device vrfs list
        dest_device: the destination device vrfs list

        when the destination vrf does not exist in the source device,
        it is cataloged as new
        """  # noqa
        df = pd.DataFrame(
            [
                [
                    source_device,
                    '',
                    '',
                    '',
                    '',
                    '',
                    dest.device,
                    dest.ip_address,
                    dest.vrf,
                    dest.interface,
                    dest.mac,
                    dest.interface_ip_address,
                    0,
                    int(dest.percent),
                    dest.validate
                ]
            ],
            columns=self.columnsDataFrame
        )

        return df

    def compareVrfs(self, dest, source) -> pd.DataFrame():
        # --
        # data frame que contendra la nueva estructura del reporte final baseline ping # noqa
        data = pd.DataFrame(
            columns=self.columnsDataFrame
        )

        # --
        # lista de dispositivos agrupados por device, source
        distinct_devices = \
            dest.groupby(['device', 'source'])['device'].count()

        for row in distinct_devices.index:
            device = row[0]
            source_device = row[1]

            # --
            # se filtran solo las vrfs del dispositivo origen
            list_source_filter = source.loc[source['device'] == source_device]

            # --
            # se filtran solo las vrfs del dispositivo destino
            list_dst_filter = dest.loc[dest['device'] == device]

            # --
            # listado de vrf del dispositivo destino
            list_unique_vrf_dst = list_dst_filter['vrf'].unique()

            # --
            # listado de vrf del dispositivo origen
            list_unique_vrf_src = list_source_filter['vrf'].unique()

            # --
            # listado completo de las vrfs de los dispositivos
            union_vrfs = \
                (set((list(list_unique_vrf_src) + list(list_unique_vrf_dst))))

            # --
            # las vrf que solo existen en el destino son categorizadas [nuevas = NEW] # noqa
            # las vrf que existen en origen, destino con los mismos parametros con [OK] # noqa
            # las vrf que existen en origin, destino con diferentes parametros con [FAIL] # noqa
            for vrf in union_vrfs:
                # --
                # segun la vrf unica del destino se filtra la vrf del origen # noqa
                list_vrf_source = list_source_filter.loc[list_source_filter['vrf'] == vrf]  # noqa

                # --
                # segun la vrf unica del destino se filtra su propio dispositivo # noqa
                list_vrf_dest = list_dst_filter.loc[list_dst_filter['vrf'] == vrf]  # noqa

                # --
                # se buscan las vrf destino en el origen
                for _, item in list_vrf_dest.iterrows():
                    vrf_item_source = list_vrf_source.loc[
                        (list_vrf_source['vrf'] == item.vrf)
                        & (list_vrf_source['interface'] == item.interface)
                        & (list_vrf_source['interface_ip_address'] == item.interface_ip_address)  # noqa
                    ]  # noqa

                    if vrf_item_source.empty:  # noqa
                        item['validate'] = 'NEW'
                        try:
                            data = pd.concat(
                                [data, self.createDataframeDestNew(item, source_device)])  # noqa
                        except Exception:
                            # si no hay datos en ambos data frame no es valido el # noqa
                            # catalogado como new (vacio)
                            pass
                    else:
                        if item.percent >= vrf_item_source.percent.values[0]:
                            item['validate'] = 'OK'
                        else:
                            item['validate'] = 'FAIL'

                        # se crea dataframe con la vrf destino, como la origen
                        data = pd.concat(
                            [data, self.createDataframeSourceDest(vrf_item_source, item)])  # noqa

            # --
            # El ultimo proceso es buscar todas las vrf origin que no existan en destino # noqa
            for vrf_source in list_unique_vrf_src:
                # --
                # segun la vrf unica del origin se filtra asi misma
                list_vrf_source = \
                    list_source_filter.loc[list_source_filter['vrf']
                                           == vrf_source]

                # --
                # segun la vrf unica del origen se filtran las destino
                list_vrf_dest = list_dst_filter.loc[list_dst_filter['vrf'] == vrf_source]  # noqa

                # --
                # se buscan las vrf origen para que no existan en el destino
                for _, item in list_vrf_source.iterrows():
                    vrf_item = list_vrf_dest.loc[
                        (list_vrf_dest['vrf'] == item.vrf)
                        & (list_vrf_dest['interface'] == item.interface)
                        & (list_vrf_dest['interface_ip_address'] == item.interface_ip_address)  # noqa
                    ]  # noqa

                    if vrf_item.empty:  # noqa
                        item['validate'] = 'FAIL'
                        data = pd.concat(
                            [data, self.createDataframeSourceOnly(item, device)])  # noqa

        return data

    def data_json(self):
        # --
        # se transforma el dataframe en un json
        self.data.to_json(f'reporte_vrf.json', orient='records')  # noqa
        return self.data

    def data_excel(self, file_name):
        # --
        # se transforma el dataframe en un excel
        try:
            self.data.to_excel(f'{file_name}.xlsx', index=False)
        except Exception:
            return False

        return True

    def run(self) -> None:
        # --
        # load the files, to precess
        self.loadFiles()

        # --
        # transform the json vrf source to dataframe
        self.df_source = self.transform(self.data_src, self.file_src_ext)

        # --
        # transform the json vrf destination to dataframe
        self.df_dest = self.transform(self.data_dst, self.file_dst_ext)

        # --
        # compare the vrfs
        self.data = self.compareVrfs(self.df_dest, self.df_source)



import os
from pathlib import Path
from typing import Optional

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer  # noqa

try:
    from .c_records import Records
except Exception:
    from c_records import Records


@app.command()
def records(
    command: Optional[Path] = typer.Option(
        ...,
        help="The command for filter information en DB",
    )
):
    """
    Data for all device filter with command
    """

    if command:
        print(Records.findData(command))

    
    





from typing import Optional
import os

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer  # noqa

try:
    from .c_merge_ping import PingMerge
except Exception:
    from c_merge_ping import PingMerge  # noqa


# --
# mantiene el estado de los comandos
state = {}


def callback_file_extension(value: str):
    ext = os.path.splitext(value)[1]

    if ext in [".xlsx", ".xls", ".json", ".csv"]:
        return value
    else:
        raise typer.BadParameter(
            "The file format must be .xlsx, .xls, .json o .csv"
        )


def callback_output(value: str):
    if value not in ['json', 'excel']:
        raise typer.BadParameter("Only json or excel type is valid",
                                 param_hint="--output")

    state['output'] = value
    return value


def callback_name(value: str):
    if state['output'] == "excel":
        if value is None or value == "":
            raise typer.BadParameter(
                "a name has not been defined for the excel file",
                param_hint="--name")
    return value


@app.command()
def ping_merge(
    file_src: Optional[str] = typer.Option(
        ...,
        help="Vrf origin listing file",
        callback=callback_file_extension
    ),
    file_dst: Optional[str] = typer.Option(
        ...,
        help="Target vrf listing file",
        callback=callback_file_extension
    ),
    output: Optional[str] = typer.Option(
        ...,
        help="The type to print report",
        callback=callback_output
    ),
    name: Optional[str] = typer.Option(
        "",
        help="The name of the excel file",
        callback=callback_name
    ),

):
    """
    Command to merge the source vrf listing files and
    destination with validated report
    """
    ping = PingMerge(file_src, file_dst)
    ping.run()

    # --
    # define the output type for the report
    # --
    if output == "json":
        print(ping.data_json())

    if output == "excel":
        status = ping.data_excel(file_name=name)
        if status is False:
            return "excel not created."
        print(ping.data_json())



# clase para listado de vrf destino de la migracion
import json
import sys

from netmiko import ConnectHandler
from nornir.core.task import Result, Task
# from tabulate import tabulate
from tqdm import tqdm
from ttp import ttp

try:
    from .c_nornir import DevopsNornir
except Exception:
    from c_nornir import DevopsNornir

try:
    from .c_ping_dst_excel import ExcelVrfDestination
except Exception:
    from c_ping_dst_excel import ExcelVrfDestination  # noqa

try:
    from .c_templates import StringTemplates
except Exception:
    from c_templates import StringTemplates


class DestinationNetwork():

    def __init__(self, path: str = None, filter: str = None, workers: int = 2):
        nornir = DevopsNornir(path=path, filter=filter, workers=workers)
        self.nr = nornir.init()
        self.n_devices = len(self.nr.inventory.hosts.keys())
        self.template = StringTemplates()

        try:
            self.equitativo = int(100 / int(self.n_devices))
        except Exception:
            self.equitativo = 100
        self.up = self.equitativo if self.equitativo % 2 == 0 else self.equitativo + 1  # noqa
        self.indent = 3
        self.sort_keys = True
        self.data = None

    def update_pbar(self):
        self.pbar.update(self.up)
        self.pbar.refresh()
        sys.stdout.flush()

    def data_json(self):
        data = json.dumps(self.data, indent=self.indent,
                          sort_keys=self.sort_keys)
        return data

    def data_excel(self, name):
        try:
            excel = ExcelVrfDestination()
            excel.create(self.data)
            excel.save(name)
        except Exception:
            return False

        return True

    def data_table(self) -> list:
        # data = (tabulate(array_for_tabulate, headers='firstrow', tablefmt='grid'))  # noqa
        return []

    def get_vrf(self, device) -> list:

        result = []

        try:
            comando = "show vrf all | ex import | ex export"
            dato_cli = device.send_command(comando)
            parser = ttp(data=dato_cli,
                         template=self.template.get_ttp_vrf_dst())
            parser.parse()
            result = parser.result()[0][0]
        except Exception:
            pass

        return result

    def get_ip_arp_vrf(self, device, datos):

        for dato in datos:
            ips_arp = {}
            try:
                comando = f"show arp vrf {dato['vrf_name']} | i Dynamic"  # noqa
                dato_cli = device.send_command(comando)
                parser = ttp(data=dato_cli,
                             template=self.template.get_ttp_arp_vrf_dst())
                parser.parse()
                ips_arp = parser.result()[0][0]
            except Exception as e:
                print(f"error arp vrf: {e}")

            dato['ips_arp'] = ips_arp

        return datos

    def get_ping_ip_arp_vrf(self, device, datos):

        for vrf in datos:
            try:
                ips_arp = vrf['ips_arp']
            except Exception:
                ips_arp = []

            for ip in ips_arp:

                ping_arp = {}
                try:
                    comando = f"ping vrf {vrf['vrf_name']} {ip['ip_address']} count 4"  # noqa
                    dato_cli = device.send_command(comando)
                    parser = ttp(data=dato_cli,
                                 template=self.template.get_ttp_ping_ip_arp())
                    parser.parse()
                    ping_arp = parser.result()[0][0]
                except Exception:
                    pass

                ip['ping_arp'] = ping_arp

        return datos

    def ping_dst(self, task: Task):
        try:
            platform = "cisco_xr" if task.host.platform == "iosxr" else task.host.platform  # noqa
            host = {
                "ip": task.host.hostname,
                "username": task.host.username,
                "password": task.host.password,
                'port': task.host.port,
                "device_type": platform,
            }

            # [paso 01] apertura de la unica sesion por equipo
            try:
                device = ConnectHandler(**host)
            except Exception:
                try:
                    device = ConnectHandler(**host)
                except Exception:
                    device = ConnectHandler(**host)

            # [paso 02] se optiene una lista de las vrf del equipo con
            # las interfaces asignadas
            datos = self.get_vrf(device)

            # [paso 03] por cada vrf se lista la tabla de arp vrf
            datos = self.get_ip_arp_vrf(device, datos)

            # [paso 04] por cada ip se realiza un ping
            datos = self.get_ping_ip_arp_vrf(device, datos)

            result = Result(
                host=task.host,
                result={
                    "name": task.host.name,
                    "hostname": task.host.hostname,
                    "platform": task.host.platform,
                    "source": task.host["source"],
                    "list_vrf": datos
                },
            )

        except Exception as e:
            result = Result(
                host=task.host,
                result={"error": str(e)}
            )

        self.update_pbar()

        return result

    def run(self):
        dict_result = {}

        # solo si hay dispositivos en el inventario
        if self.n_devices > 0:
            self.pbar = tqdm(total=100)

            result = self.nr.run(
                name="ping destination",
                task=self.ping_dst,
            )

            self.pbar.close()

            for item in result:
                dict_result[item] = {"data": result[item][0].result}

        self.data = dict_result

        return dict_result



import json
import sys

from napalm import get_network_driver
from nornir.core.task import Result, Task
from tabulate import tabulate
from tqdm import tqdm

from .c_database import db_conecction as dbase


class Records:

    def findData(command):
        try:
            db = dbase()

            info = db["db_cibercca"]

            # Apply filter for data         
            filter = {'command': str(command)}
            infoReceived = list(info.find(filter))

            for item in infoReceived:
                if '_id' in item:
                    item['data_from_device'] = json.loads(item['data_from_device'])
                    del item['_id']

            documentos_json = json.dumps(infoReceived, default=str, indent=4)

            return documentos_json

        except:
            return print('Do not exist data in Database')



import os
from pathlib import Path
from typing import Optional

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer  # noqa

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

try:
    from .c_ping_src import SourceNetwork
except Exception:
    from c_ping_src import SourceNetwork


try:
    from .c_ping_dst import DestinationNetwork
except Exception:
    from c_ping_dst import DestinationNetwork  # noqa


try:
    from .c_typer_callback import CallbackCommand
except Exception:
    from c_typer_callback import CallbackCommand  # noqa


# estado de los comandos
state = {}


def callback_path(value: Path):  # noqa
    if value.is_dir():
        return value
    raise typer.BadParameter("inventory is not directory",
                             param_hint="--path")


def callback_output(value: str):
    if value not in ['json', 'excel']:
        raise typer.BadParameter("Only json or excel type is valid",
                                 param_hint="--output")

    state['output'] = value
    return value


def callback_name(value: str):
    if state['output'] == "excel":
        if value is None or value == "":
            raise typer.BadParameter(
                "a name has not been defined for the excel file",
                param_hint="--name")
    return value


def callback_process(process: str):
    if process not in ['src', 'dst']:
        raise typer.BadParameter("Only src or dst process",
                                 param_hint="--process")
    state['process'] = process
    return process


def callback_input(input: str):
    if state["process"] == "dst":
        if input is None or input == "":
            raise typer.BadParameter("no input file defined",
                                     param_hint="--input")

        extension = os.path.splitext(input)[1]
        if extension not in ['.json', '.excel']:
            raise typer.BadParameter("Only json or excel input for process dst",  # noqa
                                     param_hint="--input")
    return input


@app.command()
def ping(
    path: Optional[Path] = typer.Option(
        ...,
        help="The path to inventory",
        callback=callback_path
    ),
    group: Optional[str] = typer.Option(
        ...,
        help="The groups to filter inventory"
    ),
    workers: Optional[int] = typer.Option(
        2,
        help="The parallel execution"
    ),
    output: Optional[str] = typer.Option(
        "json",
        help="The type to print report",
        callback=callback_output
    ),
    name: Optional[str] = typer.Option(
        "",
        help="The name of the excel file",
        callback=callback_name
    ),
    process: Optional[str] = typer.Option(
        ...,
        help="what type of process for the vrf report [src, dst]",
        callback=callback_process
    ),
):
    """
    report por vrf and ping results for inventory devices
    """

    # --
    # define the process type for the report
    # --
    if process == "src":
        data = SourceNetwork(path=path, filter=group, workers=workers)
        data.run()

    if process == "dst":
        data = DestinationNetwork(path=path, filter=group, workers=workers)
        data.run()

    # --
    # define the output type for the report
    # --
    if output == "json":
        print(data.data_json())

    if output == "excel":
        status = data.data_excel(name=name)
        if status is False:
            return "excel not created."
        print(data.data_json())



import sys
import os
from pathlib import Path
from typing import Optional
from .c_alive import Alive

try:
    from .c_typer import app, typer
except Exception:
    from .c_typer import app, typer  # noqa

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

@app.command()
def alive(
    path: Optional[Path] = typer.Option(
        ...,
        help="The path to inventory",
    ),
    group: Optional[str] = typer.Option(
        ...,
        help="The groups to filter inventory"
    ),
    workers: Optional[int] = typer.Option(
        2,
        help="The parallel execution"
    ),
    output: Optional[str] = typer.Option(
        "json",
        help="The type to print report",
    ),
):
    """
    Alive for all device filter with groups
    """

    if path is None:
        path_inventory = BASE_DIR + "/inventory/"
    else:
        path_inventory = path

    alive = Alive(path=path_inventory, filter=group, workers=workers)
    alive.run()

    if output == 'json':
        print(alive.data_json())

    if output == 'table':
        print(alive.data_table())
    
    if output == 'database':
        print(alive.data_database())
    



import os
import platform

try:
    from .c_typer import app, typer
except Exception:
    from .c_typer import app, typer

BASE_DIR = os.path.dirname(os.path.abspath(__file__))


@app.command()
def inventory(
    create: bool = typer.Option(
        "",
        help="create files from inventory examples",
    ),
):
    """
    Create files for inventory system
    """
    system = platform.system()

    if create:
        exists = os.path.exists("./inventory/")
        if not exists:
            if system == "Windows":
                os.system(f"xcopy /E /I \"{BASE_DIR}\\inventory\" .\\inventory")
            elif system in ["Linux", "Darwin"]:
                os.system(f"cp -r {BASE_DIR}/inventory/ .")
            else:
                print(f"Unsupported OS: {system}")
        else:
            print("Inventory already exists")



ttp_template_services = """
<group name="services">
 service instance {{service_instance}} {{type_instance}}
  <group name="data">
  description {{description | re(".+")}}
  </group>
  <group name="data">
  encapsulation dot1q {{dot1q}}
  </group>
  <group name="data">
  rewrite ingress tag pop 1 {{rewrite_ingress}}
  </group>
  <group name="data">
  xconnect {{xconnect_ip}} {{xconnect_id}} {{xconnect_class}} {{xconnect_class_data}}
  </group>
  <group name="data">
  service-policy input {{service_policy_input}}
  </group>
  <group name="data">
  service-policy output {{service_policy_output}}
  </group>
  <group name="data">
  bridge-domain {{bridge_domain}}
  </group>
</group>
"""  # noqa


ttp_template_vlan_service = """
<group name="interface_vlan">
interface {{interface_vlan}}
 <group name="description">
 description {{description | re(".+")}}
 </group>
 <group name="vrf">
 ip vrf forwarding {{vrf}}
 </group>
 <group name="xconnect">
 xconnect {{xconnect_ip}} {{xconnect_id}} {{xconnect_class}} {{xconnect_class_data}}
 </group>
 <group name="ip_address_primary">
 ip address {{ip_address}} {{mask}}
 </group>
 <group name="ip_address_secundary">
 ip address {{ip_address}} {{mask}} {{type}}
 </group>
 <group name="ip_access_group">
 ip access-group {{access_group}} {{type}}
 </group>
 <group name="service_policy_input">
 service-policy input {{service_policy_input}}
 </group>
 <group name="service_policy_output">
 service-policy output {{service_policy_output}}
 </group>
 <group name="ip_helper_address">
 ip helper-address {{ip_helper_address}}
 </group>
 <group name="state">
 {{state}}
 </group>
</group>
"""  # noqa


ttp_template_bridge = """
<group name="bridges">
State: {{state}}                    Mac learning: {{mac_learning}}
    <group name="interfaces">
    {{interface}} {{type}} {{ins}} {{bridge_domain}}
    </group>
</group>
"""  # noqa

# --
# rename for save old version template
ttp_template_ports_old_version = """
<group name="ports">
{{num}}  {{vlan}}  {{mac_address}}   {{type}}  {{learn}}          {{age}}   {{port | re(".+")}}
</group>
"""  # noqa

ttp_template_ports = """
          Mac Address Table
-------------------------------------------

Vlan    Mac Address       Type        Ports
----    -----------       --------    -----
<group name="ports">
 {{vlan}}    {{mac_address}}    {{type}}     {{port | ORPHRASE}}
</group>
"""  # noqa


ttp_template_trunk = """
<group name="services">
 <group name="data">
 switchport trunk encapsulation {{dot1q}}
 </group>
 <group name="data">
 switchport trunk allowed vlan {{switchport_vlans | | split(',')}}
 </group>
 <group name="data">
 switchport mode {{switchport_mode}}
 </group>
 <group name="data">
 mls {{mls}}
 </group>
</group>
"""


ttp_template_interfaces = """
<group name="interfaces">
{{interface}} is {{state | re(".+")}}, line protocol is {{protocol}} 
  Hardware is {{type_hardware}}, address is {{mac_address}} (bia {{bia_mac_address}})
  Description: {{description | re(".+")}}
  MTU {{mtu}} bytes, BW {{speed}} Kbit/sec, DLY 10 usec, 
</group>
"""  # noqa


ttp_template_interfaces_descriptions = \
"""
<group name="interfaces">
{{interface}}                         {{state}}             {{protocol}}       {{description | re(".+")}}
</group>
"""  # noqa


ttp_template_interaces_data = \
"""
<group>
{{name}} is {{ state | re(".+") }}, line protocol is {{protocol | re(".+") }} 
  <group name='hardware'>
  Hardware is {{type_hardware | re(".+")}}, address is {{mac_address}} (bia {{bia}})
  </group>
  <group name='mtu'>
  MTU {{mtu}} bytes, BW {{bw}} {{speed_type_bw}}, DLY {{dly}} usec, 
  </group>
</group>
"""  # noqa


ttp_template_multiple_vrf = \
"""
<group>
  {{vrf_name}}                              {{default_rd}}            {{protocols}}        {{interface}}
    <group name='interfaces'>
                                                                   {{interface}}
    </group>
  {{_end_}}
</group>
"""  # noqa

ttp_template_arp_vrf = \
"""
<group>
{{protocol}}  {{ip_address}}            {{age}}   {{mac}}  {{type}}   {{interface}}
</group>
<group>
{{protocol}}  {{ip_address}}            {{age}}   {{mac}}  {{type}}
</group>
"""  # noqa


ttp_template_arp_vrf_dst = \
"""
<group>
{{ip_address}}    {{age}}   {{mac}}  {{protocol}}    {{type}}  {{interface}}
</group>
"""  # noqa


ttp_template_ping_ip_arp = \
"""
<group>
Success rate is {{percent}} percent ({{rate}}/{{repeat}}){{extra | _line_}}
</group>
<group>
Success rate is {{percent}} percent ({{rate}}/{{repeat}})
</group>
"""  # noqa


ttp_template_vrf_dst = \
"""
<group>
{{vrf_name}}                {{rd}}
    <group name='import'>
                                         import  {{rt}}         {{afi}}  {{safi}}
    </group>
    <group name='export'>
                                         export  {{rt}}         {{afi}}  {{safi}}
    </group>
  {{_end_}}
</group>
"""  # noqa


class StringTemplates:

    def get_ttp_services(self):
        global ttp_template_services
        return ttp_template_services

    def get_ttp_trunk(self):
        global ttp_template_trunk
        return ttp_template_trunk

    def get_ttp_vlan(self):
        global ttp_template_vlan_service
        return ttp_template_vlan_service

    def get_ttp_bridge(self):
        global ttp_template_bridge
        return ttp_template_bridge

    def get_ttp_ports(self):
        global ttp_template_ports
        return ttp_template_ports

    def get_ttp_template_interfaces(self):
        global ttp_template_interfaces
        return ttp_template_interfaces

    def get_ttp_template_interfaces_descriptions(self):
        global ttp_template_interfaces_descriptions
        return ttp_template_interfaces_descriptions

    def get_ttp_template_interaces_data(self):
        global ttp_template_interaces_data
        return ttp_template_interaces_data

    def get_ttp_vrf(self):
        global ttp_template_multiple_vrf
        return ttp_template_multiple_vrf

    def get_ttp_arp_vrf(self):
        global ttp_template_arp_vrf
        return ttp_template_arp_vrf

    def get_ttp_ping_ip_arp(self):
        global ttp_template_ping_ip_arp
        return ttp_template_ping_ip_arp

    def get_ttp_vrf_dst(self):
        global ttp_template_vrf_dst
        return ttp_template_vrf_dst

    def get_ttp_arp_vrf_dst(self):
        global ttp_template_arp_vrf_dst
        return ttp_template_arp_vrf_dst



from openpyxl import Workbook
from openpyxl.styles import Font, PatternFill


class ExcelVrfSource():

    def __init__(self):
        self.wb = Workbook()
        self.ws = self.wb.create_sheet("interfaces", 0)
        self.font_header = Font(color='ffffff', bold=True)
        self.index = 2
        self.na = "N/A"

    def setHeadCol(self, ws, col, name):
        self.ws[col] = name
        self.ws[col].fill = PatternFill(
            start_color='000000', end_color='000000', fill_type="solid")
        self.ws[col].font = self.font_header
        return ws

    def create(self, data):

        self.setHeadCol(self.ws, 'A1', "device")
        self.setHeadCol(self.ws, 'B1', "hostname")
        self.setHeadCol(self.ws, 'C1', "vrf")
        self.setHeadCol(self.ws, 'D1', "learning interface")
        self.setHeadCol(self.ws, 'E1', "neighbor mac address")
        self.setHeadCol(self.ws, 'F1', "arp neighbor")
        self.setHeadCol(self.ws, 'G1', "ping result")

        for key in data:
            device = data[key]['data']

            EC_DEVICE_NAME = device['name']
            EC_DEVICE_HOSTNAME = device['hostname']

            # --
            # listar las vrfs asignadas al equipo origen
            # --

            for vrf in device['list_vrf']:
                EC_VRF_NAME = vrf['vrf_name']

                if len(vrf['ips_arp']) > 0:
                    for ip in vrf["ips_arp"]:
                        EC_VRF_INTERFACE = ip['interface']
                        EC_VRF_MAC_ADDRESS = ip['mac']
                        EC_VRF_ARP_NEIGBORD = ip['ip_address']
                        EC_VRF_PING_PERCENT = ip["ping_arp"][0]['percent']

                        self.ws['A' + str(self.index)] = EC_DEVICE_NAME
                        self.ws['B' + str(self.index)] = EC_DEVICE_HOSTNAME
                        self.ws['C' + str(self.index)] = EC_VRF_NAME
                        self.ws['D' + str(self.index)] = EC_VRF_INTERFACE
                        self.ws['E' + str(self.index)] = EC_VRF_MAC_ADDRESS
                        self.ws['F' + str(self.index)] = EC_VRF_ARP_NEIGBORD
                        self.ws['G' + str(self.index)] = EC_VRF_PING_PERCENT

                        self.index += 1
                else:
                    self.ws['A' + str(self.index)] = EC_DEVICE_NAME
                    self.ws['B' + str(self.index)] = EC_DEVICE_HOSTNAME
                    self.ws['C' + str(self.index)] = EC_VRF_NAME
                    self.ws['D' + str(self.index)] = self.na
                    self.ws['E' + str(self.index)] = self.na
                    self.ws['F' + str(self.index)] = self.na
                    self.ws['G' + str(self.index)] = self.na

                    self.index += 1

    def save(self, name):
        self.wb.save(f"{name}.xlsx")



from pymongo import MongoClient
import certifi
import yaml

with open("./inventory/dbconnect.yaml", "r") as file:
    config = yaml.safe_load(file)
    
uri = config['database']['mongodbUri']
ca = certifi.where()

def db_conecction():
    try:
        client = MongoClient(uri, tlsCAFile=ca)
        db = client["db_cibercca"] 
        print("You successfully connected to MongoDB!")
    except ConnectionError:
        print("Connection error with Database")
    return db
